import time
import cv2
import numpy as np
import chainer
import glob
import os
from chainer import serializers, optimizers, Variable, cuda
import chainer.functions as F
from yolov2 import *
from lib.utils import *
from lib.YS_image import *

# hyper parameters
train_sizes = [320, 352, 384, 416, 448]
initial_weight_file = "./backup/partial.model"
backup_path = "backup"
backup_file = "%s/backup.model" % (backup_path)
max_batches = 30000
learning_rate = 1e-5
learning_schedules = { 
    "0"    : 1e-5,
    "500"  : 1e-4,
    "10000": 1e-5,
    "20000": 1e-6 
}

lr_decay_power = 4
momentum = 0.9
weight_decay = 0.005
n_classes = 3
n_boxes = 5

# load model
print("loading initial model...")

# train model, 

yolov2 = YOLOv2(n_classes=n_classes, n_boxes=n_boxes)
model = YOLOv2Predictor(yolov2)

#serializers.load_hdf5(initial_weight_file, model)

cuda.get_device(0).use()
model.to_gpu()
model.cleargrads()

optimizer = optimizers.MomentumSGD(lr=learning_rate, momentum=momentum)
#optimizer.use_cleargrads()
optimizer.setup(model)
optimizer.add_hook(chainer.optimizer.WeightDecay(weight_decay))

# start to train
print("start training")
for batch in range(max_batches):
    if str(batch) in learning_schedules:
        optimizer.lr = learning_schedules[str(batch)]
    if batch % 80 == 0:
        input_width = input_height = train_sizes[np.random.randint(len(train_sizes))]

    #train_iter = iterators.SerialIterator(train, batch_size=100, shuffle=True)

    x, t = generate_samples(n_samples=1, width=input_width,height=input_height)
    x = Variable(x)
    x.to_gpu()

    # forward
    with chainer.using_config('train', True):
        loss = model(x, t)
        print("batch: %d     input size: %dx%d     learning rate: %f    loss: %f" % (batch, input_height, input_width, optimizer.lr, loss.data))
        print("/////////////////////////////////////")

    # backward and optimize
    #optimizer.cleargrads()
        loss.backward()
        optimizer.update()

    # save model
    if (batch+1) % 500 == 0:
        model_file = "%s/%s.model" % (backup_path, batch+1)
        print("saving model to %s" % (model_file))
        serializers.save_hdf5(model_file, model)
        serializers.save_hdf5(backup_file, model)

print("saving model to %s/yolov2_final.model" % (backup_path))
serializers.save_hdf5("%s/yolov2_final.model" % (backup_path), model)

model.to_cpu()
serializers.save_hdf5("%s/yolov2_final_cpu.model" % (backup_path), model)
